------------------------------------------------------
-- Main.lua
------------------------------------------------------
local addon, _ = ...
local version = GetAddOnMetadata(addon, "Version");

-- make sure retail version...
local retailOnly = select(4, GetBuildInfo()) > 80000
if (not retailOnly) then
	print("|cffa330c9"..addon.." "..version..": Retail Version Only!|r");
end;

local CALENDAR_FILTER_DARKMOON = _G.CALENDAR_FILTER_DARKMOON;
local ITEM_BIND_ON_PICKUP = _G.ITEM_BIND_ON_PICKUP;
local ITEM_BIND_TO_BNETACCOUNT = _G.ITEM_BIND_TO_BNETACCOUNT;
local ITEM_BIND_TO_ACCOUNT = _G.ITEM_BIND_TO_ACCOUNT;
local MERCHANT_ITEMS_PER_PAGE = _G.MERCHANT_ITEMS_PER_PAGE;
local UNKNOWN = _G.UNKNOWN;

FT_NewItemInfo = {};
FT_NewVendorInfo = {};
FT_NewVendorLocation = {};

local defaultsDB = {
	CoinFormat = FT.ICON, --< Icon, Short, Long
	NoDisplay = false,
	NoSound = false,
	Displacement = 1,
};

function initiateDB(a,b)
	if type(a) ~= "table" then return {}; end;
	if type(b) ~= "table" then b = {}; end;
	for k, v in pairs(a) do
		if type(v) == "table" then
			b[k] = initiateDB(v, b[k]);
		elseif type(b[k]) ~= type(v) then
			b[k] = v;
		end;
	end;
	return b;
end;

function cleanDB(c)
	local hold = {};
	hold.CoinFormat = c.CoinFormat;
	hold.NoDisplay = c.NoDisplay;
	hold.NoSound = c.NoSound;
	wipe(c);
	hold.Version = version;
	return hold;
end;

local FT_VersionFrame = CreateFrame("FRAME");
FT_VersionFrame:RegisterEvent("ADDON_LOADED");
function FT_VersionFrame:OnEvent(event, arg1)
	if event == "ADDON_LOADED" and arg1 == addon then
		FleecingTipDB = initiateDB(defaultsDB, FleecingTipDB)
		-- remove old db references...
		if (FleecingTipDB.NoDarkmoon or FleecingTipDB.NoDarkmoon == false) then FleecingTipDB = cleanDB(FleecingTipDB); end;
		if (FleecingTipDB.NoShowCost or FleecingTipDB.NoShowCost == false) then FleecingTipDB = cleanDB(FleecingTipDB); end;
		if (FleecingTipDB.NoRecipes or FleecingTipDB.NoRecipes == false) then FleecingTipDB = cleanDB(FleecingTipDB); end;
		if FleecingTipDB.Version == nil then
			wipe(FT_NewItemInfo);
			wipe(FT_NewVendorInfo);
			wipe(FT_NewVendorLocation);
			FleecingTipDB.Version = version;
		elseif FleecingTipDB.Version ~= version then
			wipe(FT_NewItemInfo);
			wipe(FT_NewVendorInfo);
			wipe(FT_NewVendorLocation);
			FleecingTipDB.Version = version;
		end;
		FT_VersionFrame:UnregisterEvent("ADDON_LOADED");
		if (retailOnly) then
			FTUtils.PrintOnce(addon.." "..FTUtils.LightYellow(version)..": addon Loaded.", 60);
		end;
	end;
end;
FT_VersionFrame:SetScript("OnEvent", FT_VersionFrame.OnEvent);

function FT_HookTooltip(frame)
	if (frame:GetScript("OnTooltipSetItem")) then
		frame:HookScript("OnTooltipSetItem", FT_OnTooltipSetItem);
	else
		frame:SetScript("OnTooltipSetItem", FT_OnTooltipSetItem);
	end;
end;

-- check active event at present time...
function CheckEventActive(worldEvent)
	if (not worldEvent) then return ""; end;
	
	local calendar = C_DateAndTime.GetCurrentCalendarTime();
	local month, day, year = calendar.month, calendar.monthDay, calendar.year;
	
	-- Darkmoon Faire Only
	if (worldEvent == FT.DARKMOON_FAIRE) then
		local numDayEvents = C_Calendar.GetNumDayEvents(0,day);
		for e = 1,numDayEvents do
			local event = C_Calendar.GetDayEvent(0,day,e);
			if event.title == CALENDAR_FILTER_DARKMOON then
				return "|cff00ff00"..FT.ACTIVE.."|r";
			end;
		end;
		return "|cffff0000"..FT.NO_ACTIVE.."|r";
	end;
	
	
	local WEList = {
	[FT.HOLIDAY_EVENT_CW] = {201}, 		-- Children's Week
	[FT.HOLIDAY_EVENT_Brew] = {372}, 	-- Brewfest
	[FT.HOLIDAY_EVENT_HE] = {324}, 		-- Hallow's End
	[FT.HOLIDAY_EVENT_LiitA] = {423}, 	-- Love is in the Air
	[FT.HOLIDAY_EVENT_LF] = {327}, 		-- Lunar Festival
	[FT.HOLIDAY_EVENT_HF] = {321}, 		-- Harvest Festival
	[FT.HOLIDAY_EVENT_Noble] = {181}, 	-- Noblegarden
	[FT.HOLIDAY_EVENT_MFF] = {341}, 	-- Midsummer Fire Festival
	};
	
	local monthInfo = C_Calendar.GetMonthInfo();
	local curMonth, curYear = monthInfo.month, monthInfo.year;
	local monthOffset = -12 * (curYear - year) + month - curMonth;
	local numEvents = C_Calendar.GetNumDayEvents(monthOffset, day);
--	print("numEvents = "..tostring(numEvents));
--	local event = C_Calendar.GetDayEvent(monthOffset, day, 1);
--	print("event.iconTexture = "..tostring(event.iconTexture));
--	print("event.sequenceType = "..tostring(event.sequenceType));
--	print("event.title = "..tostring(event.title));
--	local event2 = C_Calendar.GetHolidayInfo(monthOffset, day, 1);
--	print("event2.name = "..tostring(event2.name));
--	print("event2.texture = "..tostring(event2.texture));
	
	for i=1, numEvents do
		local event = C_Calendar.GetDayEvent(monthOffset, day, i);
		
--		print("event.iconTexture = "..tostring(event.iconTexture));
--		event.sequenceType = "ONGOING";
--		print("event.sequenceType = "..tostring(event.sequenceType));
		
		local listEvent = WEList[worldEvent];
--		listEvent = WEList[FT.HOLIDAY_EVENT_Noble];
		if (listEvent) then
--			print("YES!!!");
			local eventFound = event.eventID;
			if (listEvent[1] == eventFound) then
--				print("OKAY!!!");
				if (event.sequenceType == "ONGOING") then
					return "|cff00ff00"..FT.ACTIVE.."|r";
				else
					local hour = GetGameTime();
					if event.sequenceType == "END" and hour <= event.endTime.hour or
						event.sequenceType == "START" and hour >= event.startTime.hour then
						return "|cff00ff00"..FT.ACTIVE.."|r";
					end;
				end;
			else
				return "|cffff0000"..FT.NO_ACTIVE.."|r";
			end;
		else	
			return "|cffff0000"..UNKNOWN.."|r";
		end;

	end;
	return "---";
end;

-- return the real zone name for BC or WoD...
function FT_RealMapName(mapID)
	if (not mapID) then return nil; end;
	local mapDetails = C_Map.GetMapInfo(mapID);
	local mapName = mapDetails.name;
	if (mapID == 104) or (mapID == 107) then
		local mapZone = C_Map.GetMapInfo(466); -- Outland
		mapName = mapName.." ("..mapZone.name..")";
	elseif (mapID == 539) or (mapID == 550) then
		local mapZone = C_Map.GetMapInfo(572); -- Draenor
		mapName = mapName.." ("..mapZone.name..")";
	end;
	return mapName;
end;

function FT_GetVendorLocation(faction,npc)
	if (not faction) or (not npc) then return nil; end;
	local locationText = "";
	local vendorCoords = FT_VendorLocations[faction][npc];
	if ( vendorCoords ~= nil) then
		local uiD = vendorCoords[1];
		if (not uiD) then return locationText; end;
		if (uiD > 0) then
			locationText = FT_RealMapName(uiD);
		end;
	end;
	return locationText;
end;

function FT_GetVendorCoordinates(faction,npc)
	if (not faction) or (not npc) then return nil; end;
	local tex = nil;
	local vendorCoords = FT_VendorLocations[faction][npc];
	if ( vendorCoords ~= nil) then
		local Ct = "";
		local Rt = FT_VendorLocations[faction][npc][2] or "";
		if (string.len(Rt) == 4) then Rt = string.format("%.1f|r",tonumber(Rt)); end;
		local Lt = FT_VendorLocations[faction][npc][3] or "";
		if (string.len(Lt) == 4) then Lt = string.format("%.1f|r",tonumber(Lt)); end;
		if (FT_VendorLocations[faction][npc][3]) then Ct = ", "; end;
		if (FT_VendorLocations[faction][npc][2]) then
			tex = " ["..Rt..Ct..Lt.."]";
		else
			tex = "";
		end;
	end;
	return tex;
end;

function ERROR_ItemInfo(id,npc)
	local iN, _, _, _, _, _, _, _, _, _, _, _, _, _, _, _, _ = GetItemInfo(id) or "";
	FTUtils.PrintOnce(FTUtils.Red(addon.." "..version.." error: ").."Can't find any information on item ["..tostring(id).."] / '"..iN.."' :: Vendor:'"..npc.."' ? ".."Please report this on the website.", 60);
end;

function ERROR_VendorInformation(npc)
	FTUtils.PrintOnce(FTUtils.Red(addon.." "..version.." error: ").."Can't find any information on '"..npc.."' ? ".."Please report this on the website.", 60);
end;

function ERROR_VendorLocation(npc)
	FTUtils.PrintOnce(FTUtils.Red(addon.." "..version.." error: ").."Can't find location for '"..npc.."' ? ".."Please report this on the website.", 60);
end;

--function ERROR_VendorCoordinates(npc)
--	FTUtils.PrintOnce(FTUtils.Red(addon.." "..version.." error: ").."'"..npc.."' coordinates coming soon!", 60);
--end

function FT_FindVendorLocation(faction, npc)
	local vendorFound = FT_VendorLocations[faction][npc];
	if (vendorFound == nil) then
		vendorFound = FT_VendorLocations["Neutral"][npc];
		if (vendorFound) then
			faction = "Neutral";
		else
			return faction, nil;
		end;
	end;
	return faction, vendorFound;
end;

local function FT_CollectInfo(itemInspect)
	local introCollected, noteCollected, worldeventCollected, professionCollected, dropCollected;
	local colorCollected = FTUtils.FONT_COLOR;
	local price = "";
	if (itemInspect.b and itemInspect.b > 0) then
		price = FTUtils.GSC(itemInspect.b);
	end;
	if (itemInspect.i ~= nil) then
		price = price.."|cffffff00"..itemInspect.i.."|r";
	end;
	introCollected = string.format(FT.SOLD_FOR_PRICE_BY, FTUtils.Hilite(price));
	if (itemInspect.need ~= nil) then
		introCollected = string.format(FT.NEED_FOR, FTUtils.Hilite(price));
		colorCollected = FT.QUEST_COLOR;
	end;
	if (itemInspect.note ~= nil) then
		noteCollected = itemInspect.note;
	end;
	if (itemInspect.made ~= nil) then
		professionCollected = true;
	end;
	if (itemInspect.i == "Drop") then
		colorCollected = FT.DROP_COLOR;
		introCollected = FT.DROPPED_BY;
		dropCollected = true;
	end;
	if (itemInspect.i == "Quest") or (itemInspect.i == "Fished") then
		introCollected = FT.OPTAINABLE_BY;
		colorCollected = FT.QUEST_COLOR;
	end;
	if (itemInspect.we ~= nil) then
		worldeventCollected = itemInspect.we;
		colorCollected = FT.SPECIAL_VENDOR_COLOR;
	end;
	if (itemInspect.i == "Useless") then
		colorCollected = FT.USELESS_COLOR;
	end;
	return introCollected, colorCollected, noteCollected, worldeventCollected, professionCollected, dropCollected;
end

function FT_OnTooltipSetItem(self)
	if (not retailOnly) then return; end;
	if (MerchantFrame:IsVisible()) then
		for buttonIndex = 1, MERCHANT_ITEMS_PER_PAGE do
			local button = _G["MerchantItem"..buttonIndex.."ItemButton"];
			if (self:IsOwned(button)) then return; end;
		end;
	end;

	local name, link = self:GetItem();
	if (not link) then return; end;
	-- workaround for OnTooltipSetItem being called twice for recipes
	local _, _, _, _, _, itemType, _, _, _, _, _, _, _, _, _, _, _ = GetItemInfo(link);
	if (itemType == FT.RECIPE and self.lastLink ~= link) then
		for i = 1, self:NumLines() do
			local line = _G[self:GetName().."TextLeft"..i];
			local text = line:GetText();
			if (text and string.sub(text, 1, 1) == "\n") then 
				self.lastLink = link;
				return; 
			end;
		end;
	end;
	self.lastLink = nil;
	
	if (link) then
		self:AddLine(" ");
		local _, _, itemID  = string.find(link, "item:(%d+)");
		itemID = tonumber(itemID);
--		itemID = 128794 --#TESTING#--
		local itemInfo = FT_ItemInfo[itemID];
		if (not itemInfo) then
			itemInfo = FT_NewItemInfo[itemID];
		end
		
		if (not FleecingTipDB.NoDisplay and itemInfo) then
			local myFaction = UnitFactionGroup("player");
			
			local vendors = FTTable.Merge(FT_VendorInfo[myFaction][itemID], FT_VendorInfo["Neutral"][itemID]);
			if (FT_NewVendorInfo[myFaction] and FT_NewVendorInfo[myFaction][itemID]) then
				vendors = FTTable.Merge(vendors, FT_NewVendorInfo[myFaction][itemID]);
			end
			if (FT_NewVendorInfo["Neutral"] and FT_NewVendorInfo["Neutral"][itemID]) then
				vendors = FTTable.Merge(vendors, FT_NewVendorInfo["Neutral"][itemID]);
			end
			
			local intro, color, note, worldevent, profession, dropfound = FT_CollectInfo(itemInfo)
			if (dropfound) then myFaction = "Drops"; end;
--			print("myFaction, intro, color, note, worldevent, profession, dropfound = "..tostring(myFaction)..", "..tostring(intro)..", "..tostring(color)..", "..tostring(note)..", "..tostring(worldevent)..", "..tostring(profession)..", "..tostring(dropfound));
			local numVendors = table.getn(vendors);
			if (numVendors > 1) then
				
				self:AddLine(intro, color.r, color.g, color.b);
				for i, aVendor in pairs(vendors) do
					local locFaction, locVendor = FT_FindVendorLocation(myFaction,aVendor)
					
					if (locVendor ~= nil) or (profession and itemInfo.b) or (dropfound) then
						local vendorName = FT[aVendor] or aVendor;
						local vendorLocation = FT_GetVendorLocation(locFaction,vendorName);
						local vendorCoordinates = FT_GetVendorCoordinates(locFaction,vendorName);
						
						if (dropfound) then
							if (FT_VendorLocations[locFaction][vendorName][4] ~= nil) then
								local spec = FT_VendorLocations[locFaction][vendorName][4];
								vendorName = vendorName.." ("..spec..")";
							end;
						end;
						
						if (vendorCoordinates) ~= nil then
							self:AddDoubleLine(string.format(FT.VENDOR_LOCATION_FORMAT, vendorName, vendorLocation)..vendorCoordinates, " ", color.r, color.g, color.b);
						else
							self:AddLine(vendorName, color.r, color.g, color.b);
--							if (not profession) then
--								ERROR_VendorCoordinates(aVendor);
--							end;
						end;
						
					else
						ERROR_VendorInformation(aVendor);
						if (aVendor ==numVendors) then
							return false;
						end;
					end;
					
				end;
				
			elseif (numVendors == 1 and not itemInfo.boe) then
				local locFaction, locVendor = FT_FindVendorLocation(myFaction,vendors[1]);
				if (locVendor ~= nil) or (profession and itemInfo.b) or (dropfound) then
					
					local vendorName = FT[vendors[1]] or vendors[1];
					local vendorLocation = FT_GetVendorLocation(locFaction,vendorName);
					local vendorCoordinates = FT_GetVendorCoordinates(locFaction,vendorName);
					
					if (dropfound) then
						if (FT_VendorLocations[locFaction][vendorName][4] ~= nil) then
							local spec = FT_VendorLocations[locFaction][vendorName][4];
							vendorName = vendorName.." ("..spec..")";
						end;
					end;
					
					if (vendorCoordinates ~= nil) and (vendorName ~= "Useless") then
						if (itemInfo.need) then
							self:AddDoubleLine(intro.." "..vendorName..vendorCoordinates, " ", color.r, color.g, color.b);
						else
							self:AddLine(intro, color.r, color.g, color.b);
							self:AddDoubleLine(string.format(FT.VENDOR_LOCATION_FORMAT, vendorName, vendorLocation..vendorCoordinates), " ", color.r, color.g, color.b);
						end;
					else
						if (vendorName == "Useless") then
							self:AddLine(FT.ITEM_WARNING, color.r, color.g, color.b);
						else
							self:AddDoubleLine(intro.." "..vendorName, " ", color.r, color.g, color.b);
--							if (not profession) then
--								ERROR_VendorCoordinates(vendorName);
--							end;
						end;
					end;
					
				else
					if (not profession) then
						ERROR_VendorInformation(vendors[1]);
						return false;
					end;
				end;
				
			else
				
				local found = false;
				for _, faction in pairs({"Alliance", "Horde", "Neutral"}) do
					if (FT_VendorInfo[faction][itemID]) then
						found = true;
						break; -- get out quick! :)
					end;
				end;
				if not (found) then
					local link, _, _, _, _, _, _, _, _, _, _, _, _, _, _, _, _ = GetItemInfo(itemID) or "";
					FTUtils.PrintOnce(FTUtils.Red(addon.." "..version.." error: ").."'"..link.."'["..itemID.."] is listed but has no vendors.", 60);
					FTUtils.PrintOnce("Please report this on the mod website.", 60);
					return false;
				end;
				
			end;
			
			if (profession) then
				local color = FT.PATTERN_COLOR;
				
				local locFaction = myFaction;
--				if (itemInfo.Neutral) then locFaction = "Neutral"; end;
				
				local patternName, _, _, _, _, _, _, _, _, _, _, _, _, _, _, _, _ = GetItemInfo(itemInfo.id) or "";
				local vendorInfo = FT_VendorInfo[locFaction][itemInfo.id];
				if not (vendorInfo) then
					vendorInfo = FT_VendorInfo["Neutral"][itemInfo.id];
				end;
				local patternInfo = FT_ItemInfo[itemInfo.id];
				
				local iCost;
				if (patternInfo.i) then
					iCost = patternInfo.i; -- Different cost then gold!
				else
					if (patternInfo.b) then
						iCost = FTUtils.GSC(patternInfo.b); -- Gold cost here!w
					else
						iCost = UNKNOWN;
					end;
				end;
--				print ("vendorInfo="..tostring(vendorInfo).." locFaction="..tostring(locFaction).." itemInfo.id="..tostring(itemInfo.id));
				local noNPC = 1 or table.getn(vendorInfo);
				self:AddDoubleLine(FT.MADE_BY.." "..itemInfo.pro, " ", color.r, color.g, color.b);
				if (noNPC == 1) then
					local NPCname = vendorInfo[1];
					if (NPCname == "Trainer") then
						self:AddDoubleLine(string.format(FT.LEARN_AT, patternName), " ", color.r, color.g, color.b);
					else
						local vendorLoc;
						locFaction, vendorLoc = FT_FindVendorLocation(locFaction,NPCname);
						if (vendorLoc ~= nil) then
							local NPClocation = FT_GetVendorLocation(locFaction,NPCname);
							local NPCcoords = FT_GetVendorCoordinates(locFaction,NPCname);
							if (NPCcoords ~= nil) then
								self:AddDoubleLine(FT.BUY..string.format(FT.PATTERN_FROM, patternName, iCost), " ", color.r, color.g, color.b);
								self:AddDoubleLine(string.format(FT.VENDOR_LOCATION_FORMAT, NPCname, NPClocation..NPCcoords), " ", color.r, color.g, color.b);
							else
								self:AddDoubleLine(FT.BUY..string.format(FT.PATTERN_FROM, patternName, iCost)..NPCname..NPClocation, " ",color.r, color.g, color.b);
--								ERROR_VendorCoordinates(NPCname);
							end;
						else
							ERROR_VendorInformation(NPCname);
						end;
					end;
				else
					self:AddDoubleLine(FT.BUY..string.format(FT.PATTERN_FROM, patternName, iCost), " ", color.r, color.g, color.b);
					for numName = 1, noNPC do
						local NPCname = vendorInfo[numName];
						locFaction, vendorLoc = FT_FindVendorLocation(locFaction,NPCname);
						if (vendorLoc ~= nil) then
							local NPClocation = FT_GetVendorLocation(locFaction,NPCname);
							local NPCcoords = FT_GetVendorCoordinates(locFaction,NPCname);
							if (NPCcoords ~= nil) then
								self:AddDoubleLine(string.format(FT.VENDOR_LOCATION_FORMAT, NPCname, NPClocation..NPCcoords), " ", color.r, color.g, color.b);
							else
								self:AddLine(NPCname, color.r, color.g, color.b);
--								ERROR_VendorCoordinates(NPCname);
							end;
						else
							ERROR_VendorInformation(NPCname);
						end;
					end;
				end;
			end;
			
			if (note ~= nil) then
				self:AddLine(note, color.r, color.g, color.b);
			end;
			if (worldevent ~= nil) then
				self:AddDoubleLine(FT.WORLD_EVENT..": "..worldevent.." ("..CheckEventActive(worldevent)..")", " ", color.r, color.g, color.b);
			end;
			
			FTUtils.PlaySound(itemID, 30);
			return true;
		end;
		
		-- FactionFriend also shows tooltips for this stuff; if it's present, let it take over
--		if (FFF_Config and not FFF_Config.NoTooltip) then return; end;
	
	end;
	
end;

function FT_OnEvent(self, event, arg1)

-- In Mists of Pandaria, the game freezes if you attempt to call GetMerchantNumItems() when MERCHANT_SHOW or MERCHANT_UPDATE is called.
-- To correct this, a delay is instituted.  I found that even with .01 delay, it still doesn't freeze.  I can live with a .01 delay.
-- I do worry about the impact to performance this code might have, though.  If you know a better method, feel free to let me know.

	local delayFrame = CreateFrame("FRAME");
	local delayTotal = 0;
	function delayFrame:OnUpdate(self, elapsed)
		delayTotal = delayTotal + self;
		if delayTotal >= .01 then
			delayTotal = 0;
			delayFrame:SetScript("OnUpdate", nil);
			FT_ScanMerchant();
		end;
	end;

	if( event == "MERCHANT_SHOW" or event == "MERCHANT_UPDATE" ) then
--		FT_ScanMerchant();		-- This is where the function was called before delay was instituted.
		delayFrame:SetScript("OnUpdate", delayFrame.OnUpdate);
	end;
	
end;

function FT_OnLoad(self)

	-- Register Slash Commands
	SLASH_FLEECINGTIP1 = "/fleecingtip";
	SLASH_FLEECINGTIP2 = "/ft";
	SlashCmdList["FLEECINGTIP"] = function(msg)
		FT_ChatCommandHandler(msg);
	end

	-- Register for Events
	self:RegisterEvent("MERCHANT_SHOW");
	self:RegisterEvent("MERCHANT_UPDATE");
	
	FT_HookTooltip(GameTooltip);
	FT_HookTooltip(ItemRefTooltip);
		
end

function FT_ChatCommandHandler(msg)
	
	-- Print Help
	if ( msg == "help" ) or ( msg == "" ) then
		FTUtils.Print(addon.." "..version..":");
		FTUtils.Print("/fleecingtip (or /ft)");
		FTUtils.Print("- "..FTUtils.Hilite("help").." - Display this help list.");
		FTUtils.Print("- "..FTUtils.Hilite("[item link]").." - Show info for an item in the chat window.");
		FTUtils.Print("- "..FTUtils.Hilite("status").." - Check current settings.");
		FTUtils.Print("- "..FTUtils.Hilite("money icon").." | "..FTUtils.Hilite("short").." | "..FTUtils.Hilite("long").." - Money format display.");
		FTUtils.Print("- "..FTUtils.Hilite("sound on").." | "..FTUtils.Hilite("off").." - Found an item sound.");
		FTUtils.Print("- "..FTUtils.Hilite("show on").." | "..FTUtils.Hilite("off").." - Display 'Fleecing Tip' information in tooltip.");
		FTUtils.Print("- "..FTUtils.Hilite("config").." - Open options.");
		return;
	end
	
	if (msg == "version") then
		FTUtils.Print(addon.." "..version);
		return;
	end
	
	if ( msg == "sound on" ) or ( msg == "sound off" ) then
		FleecingTipDB.NoSound = not FleecingTipDB.NoSound
		if (FleecingTipDB.NoSound) then
			FTUtils.Print(addon.." sound is now set to "..FTUtils.Hilite("off")..".");
		else
			FTUtils.Print(addon.." sound is now set to "..FTUtils.Hilite("on")..".");
		end
		return;
	end
	
	if ( msg == "money icon" ) then
		FleecingTipDB.CoinFormat = FT.ICON;
		FTUtils.Print(addon.." money format is set to "..FTUtils.Hilite("icon")..".");
		return;
	end
	
	if ( msg == "money short" ) then
		FleecingTipDB.CoinFormat = FT.SHORT;
		FTUtils.Print(addon.." money format is set to "..FTUtils.Hilite("short")..".");
		return;
	end
	
	if ( msg == "money long" ) then
		FleecingTipDB.CoinFormat = FT.LONG;
		FTUtils.Print(addon.." money format is set to "..FTUtils.Hilite("long")..".");
		return;
	end
	
	if ( msg == "show on" ) then
		FleecingTipDB.NoDisplay = false
		FTUtils.Print(addon.." is "..FTUtils.Hilite("on")..", and showing tooltip information.");
		return;
	end
	
	if ( msg == "show off" ) then
		FleecingTipDB.NoDisplay = true
		FTUtils.Print(addon.." showing tooltip information is "..FTUtils.Hilite("off")..".");
		return;
	end
	
	if ( msg == "status" ) then
		FTUtils.Print(addon.." status:");
		FTUtils.Print("Money format is "..FTUtils.Hilite(FleecingTipDB.CoinFormat)..".");
		if (FleecingTipDB.NoSound) then
			FTUtils.Print("Sound is "..FTUtils.Hilite("off")..".");
		else
			FTUtils.Print("Sound is "..FTUtils.Hilite("on")..".");
		end
		local gotSomething;
		if (not FleecingTipDB.NoDisplay) then
			FTUtils.Print("Show "..FTUtils.Hilite("on").." Displaying tooltip information for "..addon..".");
			gotSomething = true;
		end
		if (not gotSomething) then
			FTUtils.Print("Show "..FTUtils.Hilite("off ")..addon.." is not adding any information to tooltip.");
		end
		return;
	end
	
	if ( msg == "config" ) then
		InterfaceOptionsFrame_OpenToCategory(addon); -- get over options bug...
		InterfaceOptionsFrame_OpenToCategory(addon); -- workaround to get to FT options call twice!
		return;
	end;
	
	if (msg == "test") then
		FTUtils.Print(addon..", Data Stats:");
		local itemInfoCount = 0;
		for itemID in pairs(FT_ItemInfo) do
			local found = false;
			for _, faction in pairs({"Alliance", "Horde", "Neutral"}) do
				if (FT_VendorInfo[faction][itemID]) then
					found = true;
				end
			end
			if not (found) then
				FTUtils.Print("Item ID "..itemID.." not found in FT_VendorInfo.");
			end
			itemInfoCount = itemInfoCount + 1;
		end
		FTUtils.Print("  - "..itemInfoCount.." entries in FT_ItemInfo.");
		for _, faction in pairs({"Alliance", "Horde", "Neutral"}) do
			local vendorInfoCount = 0;
			for itemID in pairs(FT_VendorInfo[faction]) do
				if (FT_ItemInfo[itemID] == nil) then
					FTUtils.Print("Item ID "..itemID.." not found in FT_ItemInfo.");
				end
				vendorInfoCount = vendorInfoCount + 1;
			end
			FTUtils.Print("  - "..vendorInfoCount.." entries in FT_VendorInfo["..faction.."].");
		end
		local dropCount = 0;
		for itemID in pairs(FT_ItemInfo) do
			if (FT_ItemInfo[itemID].i == "Drop") then
				local dropfound = false;
				local dropName = tostring(_G.UNKNOWN);
				local loc = dropName;
				for _, faction in pairs({"Alliance", "Horde", "Neutral"}) do
					if (FT_VendorInfo[faction][itemID]) then
						dropName = FT_VendorInfo[faction][itemID][1]
						loc = faction
--						print("Name: "..tostring(dropName)); -- *Debug*
						if ( FT_VendorLocations["Drops"][dropName]) then
							dropfound = true;
						end
					end
				end
				if not (dropfound) then
					FTUtils.Print("Drop '"..dropName.."' not found for ["..loc.."] "..itemID.." itemID.");
				end
				dropCount = dropCount + 1;
			end
		end
		FTUtils.Print("  - "..dropCount.." Drop entries.");
		return;
	end

	local _, _, cmd, args = string.find(msg, "^([%l%d']+) *(.*)");
	if (cmd) then cmd = string.lower(cmd); end
		
	if (args == nil or args == "") then
		args = msg;
	end
	local postedText;
	for itemLink in string.gmatch(args, "|c%x+|Hitem:[-%d:]+|h%[.-%]|h|r") do
		postedText = nil;
		local _, _, itemID  = string.find(itemLink, "item:(%d+)");
		if (itemID == nil or itemID == "") then
			FTUtils.Print("Usage: "..FTUtils.Hilite("/ft info <item link>"));
			return;
		end
		itemID = tonumber(itemID);
		FTUtils.Print(addon..": found...");
		local itemInfo = FT_ItemInfo[itemID];
		if (itemInfo) then
			local myFaction = UnitFactionGroup("player");
			local vendors = FTTable.Merge(FT_VendorInfo[myFaction][itemID], FT_VendorInfo["Neutral"][itemID]);
			if (FT_NewVendorInfo[myFaction] and FT_NewVendorInfo[myFaction][itemID]) then
				vendors = FTTable.Merge(vendors, FT_NewVendorInfo[myFaction][itemID]);
			end
			if (FT_NewVendorInfo["Neutral"] and FT_NewVendorInfo["Neutral"][itemID]) then
				vendors = FTTable.Merge(vendors, FT_NewVendorInfo["Neutral"][itemID]);
			end
			local note = itemInfo.note;
			local intro = itemLink..": "..string.format(FT.SOLD_FOR_PRICE_BY, FTUtils.GSC(itemInfo.b));

			if (vendors == nil or vendors == {}) then
				FTUtils.Print(FTUtils.Red(addon.." "..version.." error: ")..itemLink.."("..itemID..") is listed but has no vendors. Please report this at the website you downloaded from.");
				return;
			end
			FTUtils.Print(intro);
			for i, aVendor in pairs(vendors) do
				local vendorName = FT[aVendor] or aVendor;
				local vendorNPC = FT_GetVendorCoordinates(myFaction,aVendor)
				FTUtils.Print(string.format(FT.VENDOR_LOCATION_FORMAT, vendorName, vendorNPC));
			end
			if (note and note ~= "") then
				FTUtils.Print(FTUtils.Hilite(note));
			end
			postedText = 1;
		end
		
		if (not postedText) then
			FTUtils.Print("Nothing known about "..itemLink..".");
		end
		return;
	end
	if (postedText) then
		return;
	end
	-- if we made it down here, there were args we didn't understand... time to remind the user what to do.
	FT_ChatCommandHandler("help");

end

function FT_CheckMerchant(itemID)
	for merchantIndex = 1, GetMerchantNumItems() do
		local link = GetMerchantItemLink(merchantIndex);
		local _, _, merchantItemID  = string.find(link, "item:(%d+)");
		if (tonumber(merchantItemID) == itemID) then
			return true;
		end
	end
	return false;
end

function FT_ScanMerchant()
	for index = 1, GetMerchantNumItems() do
		local link = GetMerchantItemLink(index);
		if ( link ) then
			local _, _, _, _, _, itemType, subType, _, _, _, _, _, _, _, _, _, _ = GetItemInfo(link);
			if (not FT_ItemIsBoP(index)) and (not FT_ItemBlacklist(link)) and ((itemType == FT.RECIPE) or (itemType == FT.MISCELLANEOUS and ((subType == FT.PET) or (subType == FT.MOUNT))) or (itemType == FT.CONTAINER and subType == FT.BAG)) then
				local itemID = FTUtils.DecomposeItemLink(link);
				local added;
				local _, _, price, _, _, _, _, extendedCost = GetMerchantItemInfo(index);
				local priceSummmary;
				if (price and price > 0) then
				 	priceSummmary = FTUtils.GSC(price);
				else
					priceSummmary = "";
				end
				local itemSummary = "";
				if (price == 0 and extendedCost) then
					itemCount = GetMerchantItemCostInfo(index);
					if (itemCount and itemCount > 0) then
						for costIndex = 1, itemCount, 1 do
							local itemTexture, itemValue, itemLink, itemName = GetMerchantItemCostItem(index, costIndex);
							if (not itemName) then
								itemName, _, _, _, _, _, _, _, _, _, _, _, _, _, _, _, _ = GetItemInfo(itemLink);
								if (not itemName) then
									itemName = "Unknown Item";
								end
							end
							itemSummary = itemSummary .. string.format(" + %d %s", itemValue, itemName);
						end
					end
					priceSummmary = priceSummmary .. itemSummary;
					itemSummary = string.gsub(itemSummary, "^ %+ ", "");
					priceSummmary = string.gsub(priceSummmary, "^ %+ ", "");
					priceSummmary = string.gsub(priceSummmary, " %+ $", "");
					if (itemSummary == "") then
						itemSummary = nil;
					end
				end
				if (not FT_ItemInfo[itemID]) then
--					print("|cffff9900**FT ERROR** link:"..tostring(link).." itemID="..tostring(itemID).." price:"..tostring(price).." info:"..tostring(itemSummary).."|r")
					FT_NewItemInfo[itemID] = { b=price, i=itemSummary, note="" };
					added = true;
				end
				local vendorName = UnitName("npc");
				local vendorFaction = UnitFactionGroup("npc");
				if (vendorFaction ~= UnitFactionGroup("player")) then
					vendorFaction = "Neutral";
				end
				if (FT_VendorInfo[vendorFaction][itemID] == nil or FTTable.KeyOf(FT_VendorInfo[vendorFaction][itemID], vendorName) == nil) then
					FT_NewVendorInfo[vendorFaction] = FT_NewVendorInfo[vendorFaction] or {};
					FT_NewVendorInfo[vendorFaction][itemID] = FT_NewVendorInfo[vendorFaction][itemID] or {};
					if (FTTable.KeyOf(FT_NewVendorInfo[vendorFaction][itemID], vendorName) == nil) then
						table.insert(FT_NewVendorInfo[vendorFaction][itemID], vendorName);
						added = true;
					end
				end
				FT_NewVendorLocation[vendorFaction] = FT_NewVendorLocation[vendorFaction] or {};
				FT_NewVendorLocation[vendorFaction][vendorName] = FT_NewVendorLocation[vendorFaction][vendorName] or {};
				if (not FT_VendorLocations[vendorFaction][vendorName] and not FT_NewVendorLocation[vendorFaction][vendorName]) then
					local vendorZone = GetRealZoneText();
					FT_NewVendorLocation[vendorFaction][vendorName] = vendorZone;
					added = true;
				end
				if (added) then
					FTUtils.PrintOnce(string.format(addon.." %s has detected new data!  Please report the following message:  %s [%s] sold for %s by %s (%s) in %s.", version, link, itemID, FTUtils.Hilite(priceSummmary), FTUtils.Hilite(vendorName), vendorFaction, FTUtils.Hilite(GetRealZoneText())));
				end
			end
		end
	end
end

-- Attempt #2 at a BoP check, seems to work better =P
-- Version 4.2b adds a BoA check as well.
function FT_ItemIsBoP(index)
	FT_BoPCheckingTooltip:ClearLines();
	FT_BoPCheckingTooltip:SetMerchantItem(index);
	for lineNum=1, FT_BoPCheckingTooltip:NumLines() do
		if lineNum > 4 then
			return false;
		end
		local leftText = _G["FT_BoPCheckingTooltipTextLeft"..lineNum]:GetText();
		if string.find(leftText, ITEM_BIND_ON_PICKUP) then
			return true;
		elseif string.find(leftText, ITEM_BIND_TO_BNETACCOUNT) then
			return true;
		elseif string.find(leftText, ITEM_BIND_TO_ACCOUNT) then
			return true;
		end
	end
end

-- A list of items that we don't want to detect new locations for.
function FT_ItemBlacklist(link)
	local result = false
	local FT_Blacklist = {
	[4496] = true,		-- Small Brown Pouch, sold everywhere.
	[4497] = true,		-- Heavy Brown Bag, sold everywhere.
	[4498] = true,		-- Brown Leather Satchel, sold everywhere.
	[4499] = true,		-- Huge Brown Sack, sold everywhere.
--	[17200] = true,		-- Recipe: Gingerbread Cookie.  Blacklisted because otherwise it would list neutral vendors available in enemy faction cities which serves no purpose (even if you CAN buy them there.)
--	[17201] = true,		-- Recipe: Egg Nog.  Blacklisted because otherwise it would list neutral vendors available in enemy faction cities which serves no purpose (even if you CAN buy them there.)
	[20752] = true,		-- Formula: Minor Mana Oil.  Blacklisted for having a lot of vendors.
	[20753] = true,		-- Formula: Lesser Wizard Oil.  Blacklisted for having a lot of vendors.
	[20758] = true,		-- Formula: Minor Wizard Oil.  Blacklisted for having a lot of vendors.
	[21099] = true,		-- Recipe: Smoked Sagefish.  Blacklisted for having a lot of vendors.
	[21219] = true,		-- Recipe: Sagefish Delight.  Blacklisted for having a lot of vendors.
	[22200] = true,		-- Silver Shafted Arrow.  Blacklisted because it is not a pet but is detected as one.
	[22307] = true,		-- Pattern: Enchanted Mageweave Pouch.  Blacklisted for having a lot of vendors.
	[30744] = true,		-- Draenic Leather Pack, sold everywhere.
	[60335] = true,		-- Thick Hide Pack, sold everywhere.
--	[74610] = true,		-- Lunar Lantern.  Blacklisted because although the vendor is neutral, it is only purchaseable by Alliance.
--	[74611] = true,		-- Festival Lantern.  Blacklisted because although the vendor is neutral, it is only purchaseable by Horde.
	};
	local itemID = FTUtils.DecomposeItemLink(link);
	if (FT_Blacklist[itemID]) then
		result = true;
	end;
	return result
end;
